#
# Common CMake setup 
#

set(CMAKE_CXX_STANDARD 11)
set(CMAKE_CXX_STANDARD_REQUIRED ON)
set(CMAKE_INCLUDE_CURRENT_DIR ON)

#
# Version 
#
set(SPICEOPUS_VERSION_MAJOR "3") 
set(SPICEOPUS_VERSION_MINOR "0")
string(REGEX MATCH "[0123456789]+" SPICEOPUS_VERSION_REVISION "$Revision: 397 $")

#
# OS
#
if(MSVC)
	set(SPICEOPUS_OS "windows")
else()
	set(SPICEOPUS_OS "linux")
endif()

#
# Platform
#
if(CMAKE_SIZEOF_VOID_P EQUAL 8)
    if (${CMAKE_SYSTEM_PROCESSOR} STREQUAL "x86_64")
	    set(SPICEOPUS_PLATFORM "x64")
    else()
        set(SPICEOPUS_PLATFORM "arm64")
    endif()
else()
	set(SPICEOPUS_PLATFORM "x86")
endif()

add_compile_definitions(SPICEOPUS_VERSION_MAJOR=${SPICEOPUS_VERSION_MAJOR})
add_compile_definitions(SPICEOPUS_VERSION_MINOR=${SPICEOPUS_VERSION_MINOR})
add_compile_definitions(SPICEOPUS_VERSION_REVISION=${SPICEOPUS_VERSION_REVISION})
add_compile_definitions(SPICEOPUS_PLATFORM=${SPICEOPUS_PLATFORM})

message(STATUS "Spice Opus version ${SPICEOPUS_VERSION_MAJOR}.${SPICEOPUS_VERSION_MINOR}, revision ${SPICEOPUS_VERSION_REVISION}")
message(STATUS "CMAKE system processor: ${CMAKE_SYSTEM_PROCESSOR}")
message(STATUS "Platform: ${SPICEOPUS_OS} ${SPICEOPUS_PLATFORM}")
message(STATUS "Build type: ${CMAKE_BUILD_TYPE}")

add_compile_definitions(IPC_DEBUG_VIA_STDIO)

set(SPICE_OPUS_INCLUDE_DIR "${CMAKE_CURRENT_SOURCE_DIR}/include")

if(MSVC)
  # Microsoft Visual C++
  # User can generate a VS solutions which is a multi-config build system
  # We use generator expressions to handle Debug/Release
  set(CMAKE_CONFIGURATION_TYPES Release Debug)
  set(CMAKE_MSVC_RUNTIME_LIBRARY $<$<CONFIG:Debug>:MultiThreadedDebug>$<$<CONFIG:Release>:MultiThreaded>)
  add_compile_definitions(WIN32)
  add_compile_definitions(_WINDOWS)
  add_compile_definitions(_CRT_SECURE_NO_WARNINGS)
  add_compile_definitions(YY_NO_UNISTD_H)
  add_compile_definitions($<$<CONFIG:Release>:NDEBUG>)
  add_compile_options(/fp:precise /Zc:forScope /Zc:inline /Zc:wchar_t)
  add_compile_options($<$<CONFIG:Release>:/Ot>)
  add_compile_options($<$<CONFIG:Release>:/O2>)
  add_compile_options($<$<CONFIG:Debug>:/Od>)
  
  # Do not add ZERO_CHECK project to the solutiuon 
  # (will not regenerate the projects if something changes)
  # This way we do not have to add QTDIR and CRYPTOPPDIR to the environment 
  # before building the solution with Visual Studio. 
  set(CMAKE_SUPPRESS_REGENERATION on)
else()
  # GCC/Linux
  set(CMAKE_CONFIGURATION_TYPES Release Debug)
  add_compile_definitions(CM_LINUX)
  add_compile_definitions(LINUX)
  add_compile_definitions(lnxqt)
  add_compile_definitions(REENTRANT)
  add_compile_definitions($<$<CONFIG:Release>:NO_DEBUG>$<$<CONFIG:Debug>:YES_DEBUG>)
  add_compile_options(-fPIC)
  
  # For profiling
  # add_compile_options($<$<CONFIG:Debug>:-pg>)
  # add_link_options($<$<CONFIG:Debug>:-pg>)

  # Uncomment to use the optimization level from old build system 
  # add_compile_options($<$<CONFIG:Release>:-O1>)
endif()

macro(build_cm target cmprefix)
	add_custom_command(OUTPUT
			"${CMAKE_CURRENT_BINARY_DIR}/${cmprefix}_ifstmp.c"
		COMMAND ${SPICEOPUS_CMPP} -ifs
			"${CMAKE_CURRENT_SOURCE_DIR}/${cmprefix}.ifs" 
			"${CMAKE_CURRENT_BINARY_DIR}/${cmprefix}_ifstmp.c"
		MAIN_DEPENDENCY "${CMAKE_CURRENT_SOURCE_DIR}/${cmprefix}.ifs"
		COMMENT "Building code model interface for target ${target} from file ${cmprefix}.ifs"
	)
	
	add_custom_command(OUTPUT
			"${CMAKE_CURRENT_BINARY_DIR}/${cmprefix}_modtmp.c"
		COMMAND ${SPICEOPUS_CMPP} -mod
			"${CMAKE_CURRENT_SOURCE_DIR}/${cmprefix}.mod" 
			"${CMAKE_CURRENT_SOURCE_DIR}/${cmprefix}.ifs" 
			"${CMAKE_CURRENT_BINARY_DIR}/${cmprefix}_modtmp.c"
		MAIN_DEPENDENCY	"${CMAKE_CURRENT_SOURCE_DIR}/${cmprefix}.mod"
		DEPENDS "${CMAKE_CURRENT_SOURCE_DIR}/${cmprefix}.ifs" 
		COMMENT "Building code model definition for target ${target} from file ${cmprefix}.mod"
	)
	
	if(NOT DEFINED CM_SOURCES_${target}_OUTPUTS)
		set(CM_SOURCES_${target}_OUTPUTS "")
	endif()
	list(APPEND CM_SOURCES_${target}_OUTPUTS "${CMAKE_CURRENT_BINARY_DIR}/${cmprefix}_ifstmp.c")
	list(APPEND CM_SOURCES_${target}_OUTPUTS "${CMAKE_CURRENT_BINARY_DIR}/${cmprefix}_modtmp.c")
endmacro()

macro(build_cm_item_lists target)
	add_custom_command(OUTPUT
			"${CMAKE_CURRENT_BINARY_DIR}/cminfo.h"
			"${CMAKE_CURRENT_BINARY_DIR}/cmextrn.h"
			"${CMAKE_CURRENT_BINARY_DIR}/udninfo.h"
			"${CMAKE_CURRENT_BINARY_DIR}/udnextrn.h"
		COMMAND ${SPICEOPUS_CMPP} -lst ${CMAKE_CURRENT_BINARY_DIR}
		MAIN_DEPENDENCY "${CMAKE_CURRENT_SOURCE_DIR}/modpath.lst"
		DEPENDS "${CMAKE_CURRENT_SOURCE_DIR}/udnpath.lst"
		WORKING_DIRECTORY "${CMAKE_CURRENT_SOURCE_DIR}"
		COMMENT "Building cm/udn list for target ${target}"
	)
	
	if(NOT DEFINED CM_SOURCES_${target}_LISTS)
		set(CM_SOURCES_${target}_LISTS 
			"${CMAKE_CURRENT_BINARY_DIR}/cminfo.h"
			"${CMAKE_CURRENT_BINARY_DIR}/cmextrn.h"
			"${CMAKE_CURRENT_BINARY_DIR}/udninfo.h"
			"${CMAKE_CURRENT_BINARY_DIR}/udnextrn.h"
			"${CMAKE_CURRENT_SOURCE_DIR}/udnpath.lst"
		)
	endif()
endmacro()

macro(build_va target inputfile)
    get_filename_component(tmpnamebase ${inputfile} NAME_WLE)
    set(outputfile "${tmpnamebase}.osdi")
	add_custom_command(OUTPUT
			"${CMAKE_CURRENT_BINARY_DIR}/${outputfile}"
		COMMAND	"${OPENVAF_COMPILER}" "${inputfile}" -o "${CMAKE_CURRENT_BINARY_DIR}/${outputfile}"
		MAIN_DEPENDENCY "${inputfile}"
		WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}"
		COMMENT "Building osdi model from ${inputfile}"
	)

    add_custom_target(
        osdi_${tmpnamebase} ALL 
        DEPENDS "${CMAKE_CURRENT_BINARY_DIR}/${outputfile}"
    )
	
	if(NOT DEFINED OSDI_${target}_MODELS_LIST)
		set(OSDI_${target}_MODELS_LIST "")
	endif()
	list(APPEND OSDI_${target}_MODELS_LIST "${CMAKE_CURRENT_BINARY_DIR}/${outputfile}")

    set(OSDI_${target}_MODELS_LIST "${OSDI_${target}_MODELS_LIST}" CACHE INTERNAL "List of .osdi files to install." FORCE)
endmacro()
